import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.net.HttpURLConnection;
import java.net.URI;
import java.net.URL;
import java.security.InvalidKeyException;
import java.security.NoSuchAlgorithmException;
import java.text.SimpleDateFormat;
import java.util.Base64;
import java.util.Date;
import java.util.TimeZone;

import javax.crypto.Mac;
import javax.crypto.spec.SecretKeySpec;

public class FarExample {

  private static final String API_KEY = "YOUR_PUBLIC_API_KEY";
  private static final String API_SECRET = "YOUR_PRIVATE_API_KEY";
  private static final String IF_DATABASE_ID = "YOUR_FAR_DATABASE_ID";

  private static final String API_METHOD = "GET";
  private static final String API_HOST = "https://faculty180.interfolio.com/api.php";
  private static final String API_ENDPOINT = "/units";
  private static final String API_QUERY_PARAMS = "?data=detailed";
  // private static final String API_QUERY_PARAMS = "?param1=value&param2=value";


  public static void main(String[] args) {
    System.out.println("Running...");
    try {
      // Build timestamp to spec
      SimpleDateFormat dateFormatGmt = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
      dateFormatGmt.setTimeZone(TimeZone.getTimeZone("GMT"));
      String timestamp = dateFormatGmt.format(new Date());
  
      // Build the request string (for FAR use only endpoint)
      String requestString = API_ENDPOINT;
      System.out.println("requestString: " + requestString);

      // Build message to be signed
      String message = buildMessage(timestamp, requestString);
      System.out.println("message:\n" + message);

      // Generate signature using HMAC-SHA1
      String signature = generateSignature(message, API_SECRET);
      System.out.println("signature:  " + signature);

      // Build the Interfolio API authorization header
      String authHeader = buildHeader(signature);
      System.out.println("authHeader: " + authHeader);

      // Build URI object and convert to URL object
      URI apiUri = new URI(API_HOST + API_ENDPOINT + API_QUERY_PARAMS);
      URL apiUrl = apiUri.toURL();
      System.out.println("apiUrl:     " + apiUrl.toString());
      
      // Make API request
      HttpURLConnection connection = (HttpURLConnection) apiUrl.openConnection();
      connection.setRequestMethod(API_METHOD);
      connection.setRequestProperty("Timestamp", String.valueOf(timestamp));
      connection.setRequestProperty("Authorization", authHeader);
      // When calling the FAR API include the database name in the header
      connection.setRequestProperty("INTF-DatabaseID", IF_DATABASE_ID);

      // Process API response
      BufferedReader in = new BufferedReader(new InputStreamReader(connection.getInputStream()));
      String response;
      while ((response = in.readLine()) != null) {
        System.out.println(response);
      }
      in.close();
    } catch (Exception e) {
      System.err.println("error:      " + e.getMessage());
    }
    System.out.println("...all done.");
  }

  private static String buildMessage(String timestamp, String requestString) {
    return API_METHOD + "\n\n\n" + timestamp + "\n" + requestString;
  }

  private static String generateSignature(String message, String secret) throws NoSuchAlgorithmException, InvalidKeyException {
    SecretKeySpec keySpec = new SecretKeySpec(secret.getBytes(), "HmacSHA1");
    Mac mac = Mac.getInstance("HmacSHA1");
    mac.init(keySpec);
    byte[] signatureBytes = mac.doFinal(message.getBytes());
    return Base64.getEncoder().encodeToString(signatureBytes);
  }

  private static String buildHeader(String signature) {
    return "INTF " + API_KEY + ":" + signature;
  }
}

